/* ===========================================================================
|
|     (c) Copyright 1993, Hewlett-Packard NSMD, all rights reserved.
|
|     MODULE       Data Compression (Dc)
|     FILE         libdc.h
|     RCS          $Header: /src/da/dc/libdc.h /main/6 2000-09-15 10:14:00 matejk $
|     AUTHOR       Hewlett-Packard NSMD
|
|     DESCRIPTION
|        Compress/Decompress Library API
|
|     HISTORY
|         01.12.93   caka          Initial coding
|
 ========================================================================== */

#if !defined(LIBDC_H)
#define LIBDC_H

#if !defined(UCHAR)
#define UCHAR   unsigned char
#endif

#if !defined(ULONG)
#define ULONG   unsigned long
#endif


/* ---------------------------------------------------------------------------
|    
|    Module Defines - Unique Data Compression Type Identifiers
|    
 -------------------------------------------------------------------------- */

#define    DC_NONE       0			/* No data compression	*/
#define    DC_LZ43       0x0b0b0b0b		/* Lempel-Ziv 4.3	*/
#define    DC_GZIP       1			/* GNU ZIP (not impl.)	*/


typedef int (*dcCallback)(UCHAR *, ULONG);
typedef int (*dcCallbackExtended)(UCHAR *, ULONG, void*);


/* ===========================================================================
|
|     MODULE       Data Compression (Dc)
|     FUNCTION     DcInit
|
|     ARGUMENTS
|         void
|
|     RETURNS
|         DC_NONE	Initialization failed, compression disabled
|         DC_LZ43	Initialization sucessful, Lempel-Ziv compression
|	  DC_...	Customer specific & implemented compression type
|
|     DESCRIPTION
|         This function is called by the Disk Agent's initialization
|         procedure at startup. It should initialize the DC module and
|	  return an indication on what compression type will be used.
|
|     REMARKS
|         The current version od this API does not allow dynamic switching 
|	  between different compression algorithms depending on the type of
|	  input data. This might change in the future.
|
 ========================================================================== */
#if TARGET_WIN32 
__declspec(dllexport)
#endif

extern ULONG
DcInit (
#if PROTOTYPES
    void
#endif
);


/* ===========================================================================
|
|     MODULE       Data Compression (Dc)
|     FUNCTION     DcCompress
|
|     ARGUMENTS
|         readCallback		A pointer to the data input callback
|         writeCallback		A pointer to the data output callback
|
|     RETURNS
|         0		Error
|         1		OK
|
|     DESCRIPTION
|	  This  is  the  actual  stream  compression   function.  It
|	  receives its input data using the readCallback  data input
|	  method  and  writes  its  output  using the  writeCallback
|	  output method.  Here are a few guidelines the  compression
|	  module developer should keep in mind:
|
|	  1. The nature of the input and output  media is unknown to
|	     the  compression  module.  Both input and output should
|	     be  treated  as  read-only  and   write-only   streams,
|	     respectively.
|
|	  2. The DcCompress function is called once for each file to
|	     be compressed.  It should read its input until EOF (the
|	     input   handler   returns   zero)  and  make  sure  all
|	     intermediate  output  buffers  are  flushed  before  it
|	     returns.
|
|	  3. Each  call to the  input  method  is  sooner  or  later
|	     directly  translated  into a read() system call.  Thus,
|	     for better  performance, the compression  engine should
|	     try to read its input in large chunks and store them in
|	     an internal input data buffer.
|
|	  4. The  readCallback  function returns the number of bytes
|	     read.  This number can, under certain circumstances, be
|	     less than the number of bytes actually  requested.  The
|	     compression  engine  should  not treat this as an error
|	     but simply  issue a second read (this is similar to BSD
|	     sockets,    reads   may    succeed    partially).   The
|	     readCallback returns zero in case of error or at EOF.
|
|	  5. The writeCallback function always succeedes completely.
|	     If an error condition is encountered, the writeCallback
|	     never returns.
|
|	     readCallback returns:
|		>0	- Number of bytes read
|		0	- Error or EOF
|
|	     writeCallback returns:
|		>0	- number of bytes written
|		0	- Error
|
|     REMARKS
|	  Never call the readCallback to read zero bytes, as you
|	  will be returned zero which means 'Error or EOF'. Also,
|	  never attempt to write zero bytes for the same reason.
|
 ========================================================================== */
#if TARGET_WIN32 
__declspec(dllexport)
#endif

extern int
DcCompress (
#if PROTOTYPES
    dcCallback  readCallback,
    dcCallback  writeCallback
#endif
);

/* ===========================================================================
|
|     MODULE       Data Compression (Dc)
|     FUNCTION     DcCompress
|
|     ARGUMENTS
|         readCallback		A pointer to the data input callback
|         writeCallback		A pointer to the data output callback
|         externData		A pointer to the additional data.
|                           Compression library must pass this pointer
|                           to readCallback and writeCallback functions.
|
|
|     RETURNS
|         0		Error
|         1		OK
 ========================================================================== */

#if TARGET_WIN32 
__declspec(dllexport)
#endif

extern int
DcCompress2 (
#if PROTOTYPES
    dcCallbackExtended  readCallback,
    dcCallbackExtended  writeCallback,
    void                *externData
#endif
);


/* ===========================================================================
|
|     MODULE       Data Compression (Dc)
|     FUNCTION     DcDecompress
|
|     ARGUMENTS
|         readCallback		A pointer to the data input callback
|         writeCallback		A pointer to the data output callback
|
|     RETURNS
|         0		Error
|         1		OK
|
|     DESCRIPTION
|	  This  is  the  actual  stream decompression  function.  It
|	  receives its input data using the readCallback  data input
|	  method  and  writes  its  output  using the  writeCallback
|	  output method.  Here are a few guidelines the  compression
|	  module developer should keep in mind:
|
|	  1. The nature of the input and output  media is unknown to
|	     the decompression  module. Both input and output should
|	     be  treated  as  read-only  and   write-only   streams,
|	     respectively.
|
|	  2. The DcDecompress  function is called once for each file
|	     to be decompressed.  It should read its input until EOF
|	     (the  input  handler  returns  zero)  and make sure all
|	     intermediate  output  buffers  are  flushed  before  it
|	     returns.
|
|	  3. Both  input and  output  may be done in byte  chunks of
|	     arbitrary length.   The readCallback  typically returns
|	     data from an internal  OmniBack  data buffer.  However,
|	     for  performance  reasons  it is a good  idea to output
|	     larger  amounts  of data at  once,  since  calls to the
|	     writeCallback  may be directly  translated into write()
|	     system calls.
|
|	  4. The readCallback may succeed partially.  A return value
|	     of zero denotes an error or the end of the current file
|	     to be decompressed.  The writeCallback  always succeeds
|	     completely and never reports any errors back.
|
|	     readCallback returns:
|		>0	- Number of bytes read
|		0	- Error or EOF
|
|	     writeCallback returns:
|		>0	- Number of bytes written
|		0	- Error
|
|     REMARKS
|	  Never call the readCallback to read zero bytes, as you
|	  will be returned zero which means 'Error or EOF'. Also,
|	  never attempt to write zero bytes for the same reason.
|
 ========================================================================== */
#if TARGET_WIN32 
__declspec(dllexport)
#endif

extern int
DcDecompress (
#if PROTOTYPES
    dcCallback  readCallback,
    dcCallback  writeCallback
#endif
);

/* ===========================================================================
|
|     MODULE       Data Compression (Dc)
|     FUNCTION     DcDecompress
|
|     ARGUMENTS
|         readCallback		A pointer to the data input callback
|         writeCallback		A pointer to the data output callback
|         externData		A pointer to the additional data.
|                           Compression library must pass this pointer
|                           to readCallback and writeCallback functions.
|
 ========================================================================== */

#if TARGET_WIN32 
__declspec(dllexport)
#endif

extern int
DcDecompress2 (
#if PROTOTYPES
    dcCallbackExtended  readCallback,
    dcCallbackExtended  writeCallback,
    void                *externData
#endif
);


/* ===========================================================================
|
|     MODULE       Data Compression (Dc)
|     FUNCTION     DcExit
|
|     ARGUMENTS
|         void
|
|     RETURNS
|         0		Error
|         1		OK
|
|     DESCRIPTION
|         This function is called to allow the DC module to perform
|         post-processing cleanup actions. Its return value is ignored.
|
 ========================================================================== */
#if TARGET_WIN32 
__declspec(dllexport)
#endif

extern int
DcExit (
#if PROTOTYPES
    void
#endif
);


#endif /* LIBDC_H */
